@@ -13,7 +13,7 @@ use MarpaX::Languages::C::AST::Impl qw//;
 use MarpaX::Languages::C::AST::Scope qw//;
 use MarpaX::Languages::C::AST::Callback::Events qw//;
 
-our $VERSION = '0.18'; # TRIAL VERSION
+our $VERSION = '0.20'; # TRIAL VERSION
 
 
 # ----------------------------------------------------------------------------------------
@@ -99,7 +99,7 @@ sub parse {
     $self->_getLexeme(\%lexeme);
     $self->_doScope(\%lexeme);
     $self->_doEvents();
-    $pos += $self->_doPauseAfterLexeme(\%lexeme);
+    $pos += $self->_doPauseBeforeLexeme(\%lexeme);
     $self->_doLogInfo(\%lexeme);
     $self->_doLexemeCallback(\%lexeme);
     $self->_doPreprocessing($pos);
@@ -113,6 +113,15 @@ sub parse {
   return $self;
 }
 
+# ----------------------------------------------------------------------------------------
+
+sub scope {
+  my ($self) = @_;
+
+  return $self->{_scope};
+}
+
+
 # ----------------------------------------------------------------------------------------
 sub _show_last_expression {
   my ($self) = @_;
@@ -164,7 +173,9 @@ sub _doEvents {
 
   if (%events) {
     my @events = keys %events;
-    $log->debugf('[%s] Events: %s', whoami(__PACKAGE__), \@events);
+    if ($log->is_debug) {
+	$log->debugf('[%s] Events: %s', whoami(__PACKAGE__), \@events);
+    }
     $self->{_callbackEvents}->exec(@events);
   }
 }
@@ -175,7 +186,7 @@ sub _getLexeme {
   #
   # Get paused lexeme
   # Trustable if pause after
-  # See _doPauseAfterLexeme for the others
+  # See _doPauseBeforeLexeme for the others
   #
   my $lexeme = $self->{_impl}->pause_lexeme();
   if (defined($lexeme)) {
@@ -189,8 +200,10 @@ sub _getLexeme {
 sub _doLogInfo {
   my ($self, $lexemeHashp) = @_;
 
-  if (exists($lexemeHashp->{name}) && exists($self->{_logInfo}->{$lexemeHashp->{name}})) {
-    $log->infof("[%8d:%3d] %-30s %s", $lexemeHashp->{line}, $lexemeHashp->{column}, $lexemeHashp->{name}, $lexemeHashp->{value});
+  if (exists($lexemeHashp->{name}) && (exists($self->{_logInfo}->{$lexemeHashp->{name}}) || exists($self->{_logInfo}->{__ALL__}))) {
+      if ($log->is_info) {
+	  $log->infof("[%8d:%3d] %-30s %s", $lexemeHashp->{line}, $lexemeHashp->{column}, $lexemeHashp->{name}, $lexemeHashp->{value});
+      }
   }
 }
 # ----------------------------------------------------------------------------------------
@@ -234,7 +247,9 @@ sub _doPreprocessing {
 	my $preprocessorDirective = substr(${$self->{_sourcep}}, $-[2], $+[2] - $-[2]);
 	my $directive = substr(${$self->{_sourcep}}, $-[3], $+[3] - $-[3]);
 	my $lastChar = substr(${$self->{_sourcep}}, $-[4], $+[4] - $-[4]);
-	$log->debugf('Preprocessor: %s', $preprocessorDirective);
+	if ($log->is_debug) {
+	    $log->debugf('Preprocessor: %s', $preprocessorDirective);
+	}
 	#
 	# Last char is newline ?
 	#
@@ -283,13 +298,16 @@ sub _doScope {
 
     my $lexemeFormatString = "%s \"%s\" at position %d:%d";
     my @lexemeCommonInfo = ($lexemeHashp->{name}, $lexemeHashp->{value}, $lexemeHashp->{line}, $lexemeHashp->{column});
+    my $is_debug = $log->is_debug;
 
     if (defined($self->{_callbackEvents}->topic_fired_data('fileScopeDeclarator'))) {
       if ($self->{_callbackEvents}->topic_fired_data('fileScopeDeclarator')->[0] == -1) {
         #
         # This will be for next round.
         #
-        $log->debugf('[%s] fileScopeDeclarator: flagging lookup required at next round.', whoami(__PACKAGE__));
+	  if ($is_debug) {
+	      $log->debugf('[%s] fileScopeDeclarator: flagging lookup required at next round.', whoami(__PACKAGE__));
+	  }
         $self->{_callbackEvents}->topic_fired_data('fileScopeDeclarator')->[0] = 1;
 
       } elsif ($self->{_callbackEvents}->topic_fired_data('fileScopeDeclarator')->[0] == 1) {
@@ -299,39 +317,57 @@ sub _doScope {
         if ($lexemeHashp->{name} ne 'COMMA' &&
             $lexemeHashp->{name} ne 'SEMICOLON' &&
             $lexemeHashp->{name} ne 'EQUAL') {
-          $log->debugf('[%s] fileScopeDeclarator: next lexeme is %s, flagging reenterScope.', whoami(__PACKAGE__), $lexemeHashp->{name});
+	    if ($is_debug) {
+		$log->debugf('[%s] fileScopeDeclarator: next lexeme is %s, flagging reenterScope.', whoami(__PACKAGE__), $lexemeHashp->{name});
+	    }
           $self->{_callbackEvents}->topic_fired_data('reenterScope')->[0] = 1;
         }
         #
         # Flag lookup done
         #
-        $log->debugf('[%s] fileScopeDeclarator: flagging lookup done.', whoami(__PACKAGE__));
+	if ($is_debug) {
+	    $log->debugf('[%s] fileScopeDeclarator: flagging lookup done.', whoami(__PACKAGE__));
+	}
         $self->{_callbackEvents}->topic_fired_data('fileScopeDeclarator')->[0] = 0;
       }
     }
 
     if ($lexemeHashp->{name} eq 'LCURLY_SCOPE' || $lexemeHashp->{name} eq 'LPAREN_SCOPE') {
-      $log->debugf("[%s] $lexemeFormatString: entering scope.", whoami(__PACKAGE__), @lexemeCommonInfo);
+	if ($is_debug) {
+	    $log->debugf("[%s] $lexemeFormatString: entering scope.", whoami(__PACKAGE__), @lexemeCommonInfo);
+	}
       $self->{_scope}->parseEnterScope();
     } elsif ($lexemeHashp->{name} eq 'RCURLY_SCOPE' || $lexemeHashp->{name} eq 'RPAREN_SCOPE') {
       if ($self->{_scope}->parseScopeLevel == 1) {
-        $log->debugf("[%s] $lexemeFormatString: delay leaving scope.", whoami(__PACKAGE__), @lexemeCommonInfo);
+	  if ($is_debug) {
+	      $log->debugf("[%s] $lexemeFormatString: delay leaving scope.", whoami(__PACKAGE__), @lexemeCommonInfo);
+	  }
         $self->{_scope}->parseExitScope(0);
       } else {
-        $log->debugf("[%s] $lexemeFormatString: immediate leaving scope.", whoami(__PACKAGE__), @lexemeCommonInfo);
+	  if ($is_debug) {
+	      $log->debugf("[%s] $lexemeFormatString: immediate leaving scope.", whoami(__PACKAGE__), @lexemeCommonInfo);
+	  }
         $self->{_scope}->parseExitScope(1);
       }
     } else {
-      $log->debugf("[%s] $lexemeFormatString.", whoami(__PACKAGE__), @lexemeCommonInfo);
+	if ($is_debug) {
+	    $log->debugf("[%s] $lexemeFormatString.", whoami(__PACKAGE__), @lexemeCommonInfo);
+	}
       if ($self->{_scope}->parseScopeLevel == 1 && $self->{_scope}->parseDelay) {
         if (defined($self->{_callbackEvents}->topic_fired_data('reenterScope')) &&
             $self->{_callbackEvents}->topic_fired_data('reenterScope')->[0]) {
-          $log->debugf('[%s] reenterScope flag is on at scope 1.', whoami(__PACKAGE__));
+	    if ($is_debug) {
+		$log->debugf('[%s] reenterScope flag is on at scope 1.', whoami(__PACKAGE__));
+	    }
           $self->{_scope}->parseReenterScope();
-          $log->debugf('[%s] Unflagging reenterScope.', whoami(__PACKAGE__));
+	    if ($is_debug) {
+		$log->debugf('[%s] Unflagging reenterScope.', whoami(__PACKAGE__));
+	    }
           $self->{_callbackEvents}->topic_fired_data('reenterScope')->[0] = 0;
         } else {
-          $log->debugf('[%s] reenterScope flag is off at scope 1.', whoami(__PACKAGE__));
+	    if ($is_debug) {
+		$log->debugf('[%s] reenterScope flag is off at scope 1.', whoami(__PACKAGE__));
+	    }
           $self->{_scope}->doExitScope();
         }
       }
@@ -339,7 +375,7 @@ sub _doScope {
   }
 }
 # ----------------------------------------------------------------------------------------
-sub _doPauseAfterLexeme {
+sub _doPauseBeforeLexeme {
   my ($self, $lexemeHashp) = @_;
 
   my $delta = 0;
@@ -349,7 +385,7 @@ sub _doPauseAfterLexeme {
   #
   if (exists($lexemeHashp->{name})) {
       #
-      # pause start lexemes
+      # C grammar typedef/enumeration_constant/identifier ambiguity
       #
       if ($lexemeHashp->{name} eq 'TYPEDEF_NAME' ||
           $lexemeHashp->{name} eq 'ENUMERATION_CONSTANT' ||
@@ -372,7 +408,9 @@ sub _doPauseAfterLexeme {
 	  #
 	  # Push the unambiguated lexeme
 	  #
-	  $log->debugf('[%s] Pushing lexeme %s "%s"', whoami(__PACKAGE__), $newlexeme, $lexemeHashp->{value});
+	  if ($log->is_debug) {
+	      $log->debugf('[%s] Pushing lexeme %s "%s"', whoami(__PACKAGE__), $newlexeme, $lexemeHashp->{value});
+	  }
 	  if (! defined($self->{_impl}->lexeme_read($newlexeme, $lexemeHashp->{start}, $lexemeHashp->{length}, $lexemeHashp->{value}))) {
 	      my $line_columnp = lineAndCol($self->{_impl});
 	      logCroak("[%s] Lexeme value \"%s\" cannot be associated to lexeme name %s at position %d:%d.\n\nLast position:\n\n%s%s", whoami(__PACKAGE__), $lexemeHashp->{value}, $newlexeme, $lexemeHashp->{line}, $lexemeHashp->{column}, showLineAndCol(@{$line_columnp}, $self->{_sourcep}), $self->_context());
@@ -383,8 +421,8 @@ sub _doPauseAfterLexeme {
 	  # A lexeme_read() can generate an event
 	  #
 	  $self->_doEvents();
-      }
-  }
+        }
+    }
 
   return $delta;
 }
@@ -404,7 +442,7 @@ MarpaX::Languages::C::AST - Translate a C source to an AST
 
 =head1 VERSION
 
-version 0.18
+version 0.20
 
 =head1 SYNOPSIS
 
@@ -500,6 +538,10 @@ String containing lexeme value.
 
 Do the parsing. Takes as parameter the reference to a C source code. Returns $self, so that chaining with value method will be natural, i.e. parse()->value().
 
+=head2 scope($self)
+
+Returns the MarpaX::Languages::C::AST::Scope object.
+
 =head2 value($self, $optionalArrayOfValuesb)
 
 Return the blessed value. Takes as optional parameter a flag saying if the return value should be an array of all values or not. If this flag is false, the module will croak if there more than one parse tree value. If this flag is true, a reference to an array of values will be returned, even if there is a single parse tree value.
