@@ -15,7 +15,7 @@ use constant LHS_PROCESS_EVENT => '<process>';
 use constant CLOSEANYSCOPE_PRIORITY => -1000;
 use constant RESETANYDATA_PRIORITY => -2000;
 
-our $VERSION = '0.18'; # TRIAL VERSION
+our $VERSION = '0.20'; # TRIAL VERSION
 
 
 sub new {
@@ -260,7 +260,7 @@ sub _enumerationConstantIdentifier {
     my ($method, $callback, $eventsp) = @_;
 
     my $enum = lastCompleted($callback->hscratchpad('_impl'), 'enumerationConstantIdentifier');
-    $callback->hscratchpad('_scope')->parseEnterEnum($enum);
+    $callback->hscratchpad('_scope')->parseEnterEnum($enum, startAndLength($callback->hscratchpad('_impl')));
 }
 # ----------------------------------------------------------------------------------------
 sub _parameterDeclarationCheck {
@@ -275,7 +275,7 @@ sub _parameterDeclarationCheck {
     #
     my $nbTypedef = $#{$parameterDeclarationdeclarationSpecifiers};
     if ($nbTypedef >= 0) {
-	my ($line_columnp, $last_completed)  = @{$parameterDeclarationdeclarationSpecifiers->[0]};
+	my ($start_lengthp, $line_columnp, $last_completed)  = @{$parameterDeclarationdeclarationSpecifiers->[0]};
 	logCroak("[%s[%d]] %s is not valid in a parameter declaration\n%s\n", whoami(__PACKAGE__), $callback->currentTopicLevel, $last_completed, showLineAndCol(@{$line_columnp}, $callback->hscratchpad('_sourcep')));
     }
 }
@@ -294,13 +294,13 @@ sub _functionDefinitionCheck1 {
     #
     my $nbTypedef1 = $#{$functionDefinitionCheck1declarationSpecifiers};
     if ($nbTypedef1 >= 0) {
-	my ($line_columnp, $last_completed)  = @{$functionDefinitionCheck1declarationSpecifiers->[0]};
+	my ($start_lengthp, $line_columnp, $last_completed)  = @{$functionDefinitionCheck1declarationSpecifiers->[0]};
 	logCroak("[%s[%d]] %s is not valid in a function declaration specifier\n%s\n", whoami(__PACKAGE__), $callback->currentTopicLevel, $last_completed, showLineAndCol(@{$line_columnp}, $callback->hscratchpad('_sourcep')));
     }
 
     my $nbTypedef2 = $#{$functionDefinitionCheck1declarationList};
     if ($nbTypedef2 >= 0) {
-	my ($line_columnp, $last_completed)  = @{$functionDefinitionCheck1declarationList->[0]};
+	my ($start_lengthp, $line_columnp, $last_completed)  = @{$functionDefinitionCheck1declarationList->[0]};
 	logCroak("[%s[%d]] %s is not valid in a function declaration list\n%s\n", whoami(__PACKAGE__), $callback->currentTopicLevel, $last_completed, showLineAndCol(@{$line_columnp}, $callback->hscratchpad('_sourcep')));
     }
 }
@@ -316,7 +316,7 @@ sub _functionDefinitionCheck2 {
     #
     my $nbTypedef = $#{$functionDefinitionCheck2declarationSpecifiers};
     if ($nbTypedef >= 0) {
-	my ($line_columnp, $last_completed)  = @{$functionDefinitionCheck2declarationSpecifiers->[0]};
+	my ($start_lengthp, $line_columnp, $last_completed)  = @{$functionDefinitionCheck2declarationSpecifiers->[0]};
 	logCroak("[%s[%d]] %s is not valid in a function declaration specifier\n%s\n", whoami(__PACKAGE__), $callback->currentTopicLevel, $last_completed, showLineAndCol(@{$line_columnp}, $callback->hscratchpad('_sourcep')));
     }
 }
@@ -347,14 +347,14 @@ sub _declarationCheck {
 	#
 	# Take the second typedef
 	#
-	my ($line_columnp, $last_completed)  = @{$declarationCheckdeclarationSpecifiers->[1]};
+	my ($start_lengthp, $line_columnp, $last_completed)  = @{$declarationCheckdeclarationSpecifiers->[1]};
 	logCroak("[%s[%d]] %s cannot appear more than once\n%s\n", whoami(__PACKAGE__), $callback->currentTopicLevel, $last_completed, showLineAndCol(@{$line_columnp}, $callback->hscratchpad('_sourcep')));
     }
     foreach (@{$declarationCheckinitDeclaratorList}) {
-	my ($line_columnp, $last_completed, %counters)  = @{$_};
+	my ($start_lengthp, $line_columnp, $last_completed, %counters)  = @{$_};
         if (! $counters{structContext}) {
           if ($nbTypedef >= 0) {
-	    $callback->hscratchpad('_scope')->parseEnterTypedef($last_completed);
+	    $callback->hscratchpad('_scope')->parseEnterTypedef($last_completed, $start_lengthp);
           } else {
 	    $callback->hscratchpad('_scope')->parseObscureTypedef($last_completed);
           }
@@ -390,10 +390,10 @@ sub _storage_helper {
     my $rc;
     if (substr($symbol, 0, 1) eq '^') {
 	substr($symbol, 0, 1, '');
-	$rc = [ lineAndCol($callback->hscratchpad('_impl')), %counters ];
+	$rc = [ startAndLength($callback->hscratchpad('_impl')), lineAndCol($callback->hscratchpad('_impl')), %counters ];
     } elsif (substr($symbol, -1, 1) eq '$') {
 	substr($symbol, -1, 1, '');
-	$rc = [ lineAndCol($callback->hscratchpad('_impl')), $fixedValue || lastCompleted($callback->hscratchpad('_impl'), $symbol), %counters ];
+	$rc = [ startAndLength($callback->hscratchpad('_impl')), lineAndCol($callback->hscratchpad('_impl')), $fixedValue || lastCompleted($callback->hscratchpad('_impl'), $symbol), %counters ];
     }
 
     return $rc;
@@ -423,7 +423,7 @@ sub _reset_helper {
     return @rc;
 }
 # ----------------------------------------------------------------------------------------
-sub _collect_helper {
+sub _collect_and_reset_helper {
     my ($method, $callback, $eventsp, @topics) = @_;
 
     my @rc = ();
@@ -530,9 +530,9 @@ sub _register_rule_callbacks {
       } else {
         ($name, $value) = ($_, undef);
       }
-	my $event = $name . '$';
-	++$genomeEvents{$event};
-	++$rshProcessEvents{$event};
+      my $event = $name . '$';
+      ++$genomeEvents{$event};
+      ++$rshProcessEvents{$event};
       $genomeEventValues{$event} = $value;
     }
   }
@@ -583,6 +583,7 @@ sub _register_rule_callbacks {
     }
     #
     # rhs$ event will collect into rhs$ topic all Gx$ topics (created automatically if needed)
+    # and reset them
     #
     my $event = $rhs . '$';
     ++$rshProcessEvents{$event};
@@ -590,7 +591,7 @@ sub _register_rule_callbacks {
 			(
 			 description => $event,
                          extra_description => "$event [process] ",
-			 method =>  [ \&_collect_helper, keys %genomeTopicsNotToUpdate ],
+			 method =>  [ \&_collect_and_reset_helper, keys %genomeTopicsNotToUpdate ],
 			 method_mode => 'push',
 			 option => MarpaX::Languages::C::AST::Callback::Option->new
 			 (
@@ -602,25 +603,6 @@ sub _register_rule_callbacks {
 			 )
 			)
 	);
-    #
-    ## .. and reset them
-    #
-    $callback->register(MarpaX::Languages::C::AST::Callback::Method->new
-			(
-			 description => $event,
-                         extra_description => "$event [reset] ",
-			 method =>  [ \&_reset_helper, keys %genomeTopicsToUpdate ],
-			 method_mode => 'replace',
-			 option => MarpaX::Languages::C::AST::Callback::Option->new
-			 (
-			  condition => [ [ 'auto' ] ],  # == match on description
-			  topic => {%genomeTopicsToUpdate},
-			  topic_persistence => 'level',
-			  priority => 0,
-			 )
-			)
-	);
-
   }
 
   #
@@ -706,23 +688,6 @@ sub _register_rule_callbacks {
                    )
                   )
                  );
-  #  $self->register(MarpaX::Languages::C::AST::Callback::Method->new
-  #                 (
-  #                  description => $lhsResetEvent,
-  #                  method => [ \&_subFire, $hashp->{lhs}, $callback, \%lhsResetEvents, {$hashp->{lhs} . '$' => $lhsResetEvent, 'translationUnit$' => $lhsResetEvent} ],
-  #                  option => MarpaX::Languages::C::AST::Callback::Option->new
-  #                  (
-  #                   condition => [
-  #                                 [ sub { my ($method, $callback, $eventsp, $processEventsp) = @_;
-  #                                         return grep {exists($processEventsp->{$_})} @{$eventsp};
-  #                                       },
-  #                                   \%lhsResetEvents
-  #                                 ]
-  #                                ],
-  #                   priority => $hashp->{reset_priority}
-  #                  )
-  #                 )
-  #                );
 
   return $callback;
 }
@@ -741,7 +706,7 @@ MarpaX::Languages::C::AST::Callback::Events - Events callback when translating a
 
 =head1 VERSION
 
-version 0.18
+version 0.20
 
 =head1 DESCRIPTION
 
@@ -751,20 +716,6 @@ This modules implements the Marpa events callback using the very simple framewor
 
 Jean-Damien Durand <jeandamiendurand@free.fr>
 
-=head1 CONTRIBUTORS
-
-=over 4
-
-=item *
-
-Jeffrey Kegler <jkegl@cpan.org>
-
-=item *
-
-jddurand <jeandamiendurand@free.fr>
-
-=back
-
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2013 by Jean-Damien Durand.
