@@ -10,7 +10,7 @@ use Storable qw/dclone/;
 use Log::Any qw/$log/;
 use Carp qw/croak/;
 
-our $VERSION = '0.18'; # TRIAL VERSION
+our $VERSION = '0.20'; # TRIAL VERSION
 
 
 sub new {
@@ -29,13 +29,29 @@ sub new {
 }
 
 
+sub typedefPerScope {
+  my ($self) = @_;
+
+  return $self->{_typedefPerScope};
+}
+
+
+sub enumAnyScope {
+  my ($self) = @_;
+
+  return $self->{_enumAnyScope};
+}
+
+
 sub parseEnterScope {
   my ($self) = @_;
 
   # $self->condExitScope();
 
   my $scope = $self->parseScopeLevel;
-  $log->debugf('[%s] Duplicating scope %d to %d', whoami(__PACKAGE__), $scope, $scope + 1);
+  if ($log->is_debug) {
+      $log->debugf('[%s] Duplicating scope %d to %d', whoami(__PACKAGE__), $scope, $scope + 1);
+  }
   push(@{$self->{_typedefPerScope}}, dclone($self->{_typedefPerScope}->[$scope]));
 
   if (@{$self->{_enterScopeCallback}}) {
@@ -51,7 +67,9 @@ sub parseDelay {
   if (@_) {
     my $scope = $self->parseScopeLevel;
     my $value = shift;
-    $log->debugf('[%s] Setting delay flag to %d at scope %d', whoami(__PACKAGE__), $value, $scope);
+    if ($log->is_debug) {
+	$log->debugf('[%s] Setting delay flag to %d at scope %d', whoami(__PACKAGE__), $value, $scope);
+    }
     $self->{_delay} = $value;
   }
   return $self->{_delay};
@@ -94,8 +112,10 @@ sub parseExitScope {
 sub parseReenterScope {
   my ($self) = @_;
 
-    my $scope = $self->parseScopeLevel;
-  $log->debugf('[%s] Reenter scope at scope %d', whoami(__PACKAGE__), $scope);
+  my $scope = $self->parseScopeLevel;
+  if ($log->is_debug) {
+      $log->debugf('[%s] Reenter scope at scope %d', whoami(__PACKAGE__), $scope);
+  }
   $self->parseDelay(0);
 
 }
@@ -114,7 +134,9 @@ sub doExitScope {
   my ($self) = @_;
 
   my $scope = $self->parseScopeLevel;
-  $log->debugf('[%s] Removing scope %d', whoami(__PACKAGE__), $scope);
+  if ($log->is_debug) {
+      $log->debugf('[%s] Removing scope %d', whoami(__PACKAGE__), $scope);
+  }
   pop(@{$self->{_typedefPerScope}});
 
   if (@{$self->{_exitScopeCallback}}) {
@@ -126,21 +148,29 @@ sub doExitScope {
 
 
 sub parseEnterTypedef {
-  my ($self, $token) = @_;
+  my ($self, $token, $data) = @_;
+
+  $data //= 1;
 
   my $scope = $self->parseScopeLevel;
-  $self->{_typedefPerScope}->[$scope]->{$token} = 1;
+  $self->{_typedefPerScope}->[$scope]->{$token} = $data;
 
-  $log->debugf('[%s] "%s" typedef entered at scope %d', whoami(__PACKAGE__), $token, $scope);
+  if ($log->is_debug) {
+      $log->debugf('[%s] "%s" typedef entered at scope %d', whoami(__PACKAGE__), $token, $scope);
+  }
 }
 
 
 sub parseEnterEnum {
-  my ($self, $token) = @_;
+  my ($self, $token, $data) = @_;
 
-  $self->{_enumAnyScope}->{$token} = 1;
+  $data //= 1;
+
+  $self->{_enumAnyScope}->{$token} = $data;
   my $scope = $self->parseScopeLevel;
-  $log->debugf('[%s] "%s" enum entered at scope %d', whoami(__PACKAGE__), $token, $scope);
+  if ($log->is_debug) {
+      $log->debugf('[%s] "%s" enum entered at scope %d', whoami(__PACKAGE__), $token, $scope);
+  }
   #
   # Enum wins from now on and forever
   #
@@ -154,9 +184,11 @@ sub parseObscureTypedef {
   my ($self, $token, $scope) = @_;
 
   $scope //= $self->parseScopeLevel;
-  $self->{_typedefPerScope}->[$scope]->{$token} = 0;
+  $self->{_typedefPerScope}->[$scope]->{$token} = undef;
 
-  $log->debugf('[%s] "%s" eventual typedef obscured at scope %d', whoami(__PACKAGE__), $token, $scope);
+  if ($log->is_debug) {
+      $log->debugf('[%s] "%s" eventual typedef obscured at scope %d', whoami(__PACKAGE__), $token, $scope);
+  }
 }
 
 
@@ -164,9 +196,11 @@ sub parseIsTypedef {
   my ($self, $token) = @_;
 
   my $scope = $self->parseScopeLevel;
-  my $rc = (exists($self->{_typedefPerScope}->[$scope]->{$token}) && $self->{_typedefPerScope}->[$scope]->{$token}) ? 1 : 0;
+  my $rc = (exists($self->{_typedefPerScope}->[$scope]->{$token}) && defined($self->{_typedefPerScope}->[$scope]->{$token})) ? 1 : 0;
 
-  $log->debugf('[%s] "%s" at scope %d is a typedef? %s', whoami(__PACKAGE__), $token, $scope, $rc ? 'yes' : 'no');
+  if ($log->is_debug) {
+      $log->debugf('[%s] "%s" at scope %d is a typedef? %s', whoami(__PACKAGE__), $token, $scope, $rc ? 'yes' : 'no');
+  }
 
   return($rc);
 }
@@ -178,7 +212,9 @@ sub parseIsEnum {
   my $rc = (exists($self->{_enumAnyScope}->{$token}) && $self->{_enumAnyScope}->{$token}) ? 1 : 0;
 
   my $scope = $self->parseScopeLevel;
-  $log->debugf('[%s] "%s" is an enum at scope %d? %s', whoami(__PACKAGE__), $token, $scope, $rc ? 'yes' : 'no');
+  if ($log->is_debug) {
+      $log->debugf('[%s] "%s" is an enum at scope %d? %s', whoami(__PACKAGE__), $token, $scope, $rc ? 'yes' : 'no');
+  }
 
   return($rc);
 }
@@ -197,7 +233,7 @@ MarpaX::Languages::C::AST::Scope - Scope management when translating a C source
 
 =head1 VERSION
 
-version 0.18
+version 0.20
 
 =head1 SYNOPSIS
 
@@ -233,6 +269,14 @@ Please note that this module is logging via Log::Any.
 
 Instance a new object. Takes no parameter.
 
+=head2 typedefPerScope($self)
+
+Returns the list of known typedefs per scope. At the end of processing a source code, only scope 0 still exist. The output is a reference to an array, file-level scope being at index 0. At each indice, there is a reference to a hash with typedef name as a key, value is useless.
+
+=head2 enumAnyScope($self)
+
+Returns the list of known enums. Enums has no scope level: as soon as the parser sees an enum, it available at any level. The output is a reference to a hash with enumeration name as a key, value is useless.
+
 =head2 parseEnterScope($self)
 
 Say we enter a scope.
@@ -269,13 +313,13 @@ Leave current scope if delay flag is set and not yet done.
 
 Leave current scope.
 
-=head2 parseEnterTypedef($self, $token)
+=head2 parseEnterTypedef($self, $token, $data)
 
-Declare a new typedef with name $token, that will be visible until current scope is left.
+Declare a new typedef with name $token, that will be visible until current scope is left. $data is an optional user-data area, defaulting to 1 if not specified.
 
 =head2 parseEnterEnum($self, $token)
 
-Declare a new enum with name $token, that will be visible at any scope from now on.
+Declare a new enum with name $token, that will be visible at any scope from now on. $data is an optional user-data area, defaulting to 1 if not specified.
 
 =head2 parseObscureTypedef($self, $token)
 
@@ -293,20 +337,6 @@ Return a true value if $token is an enum.
 
 Jean-Damien Durand <jeandamiendurand@free.fr>
 
-=head1 CONTRIBUTORS
-
-=over 4
-
-=item *
-
-Jeffrey Kegler <jkegl@cpan.org>
-
-=item *
-
-jddurand <jeandamiendurand@free.fr>
-
-=back
-
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2013 by Jean-Damien Durand.
